library ieee,xil_defaultlib;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use xil_defaultlib.all;

entity average_tb is
end average_tb;

architecture Behavioral of average_tb is
   --vhodni signali            
   signal s_axi_aclk : std_logic := '0';
   signal s_axi_aresetn : std_logic := '0';
   signal s_axi_awaddr : std_logic_vector(3 downto 0) := (others => '0');
   signal s_axi_awprot : std_logic_vector(2 downto 0) := "000";
   signal s_axi_awvalid : std_logic := '0';
   signal s_axi_wdata : std_logic_vector(31 downto 0) := (others => '0');
   signal s_axi_wvalid : std_logic := '0';
   signal s_axi_wstrb : std_logic_vector(3 downto 0) := (others => '0');
   signal s_axi_bready : std_logic := '0';
   signal s_axi_araddr : std_logic_vector(3 downto 0) := (others => '0');
   signal s_axi_arprot : std_logic_vector(2 downto 0) := "000";
   signal s_axi_arvalid : std_logic := '0';
   signal s_axi_rready : std_logic := '0';

   --izhodni signali
   signal s_axi_awready : std_logic;
   signal s_axi_wready : std_logic;
   signal s_axi_bvalid : std_logic;
   signal s_axi_bresp : std_logic_vector(1 downto 0);
   signal s_axi_arready : std_logic;
   signal s_axi_rdata : std_logic_vector(31 downto 0);
   signal s_axi_rvalid : std_logic;
   signal s_axi_rresp : std_logic_vector(1 downto 0);

   -- perioda ure
   constant period : time := 10 ns;
   -- kontrola simulacije
   signal sim : std_logic := '1';
begin
   uut: entity xil_defaultlib.average_v1_0 port map(
	s00_axi_aclk => s_axi_aclk,
	s00_axi_aresetn => s_axi_aresetn,
	s00_axi_awaddr => s_axi_awaddr,
	s00_axi_awprot => s_axi_awprot,
	s00_axi_awvalid => s_axi_awvalid,
	s00_axi_awready => s_axi_awready,
	s00_axi_wdata => s_axi_wdata,
	s00_axi_wvalid => s_axi_wvalid,
	s00_axi_wready => s_axi_wready,
	s00_axi_wstrb => s_axi_wstrb,
	s00_axi_bvalid => s_axi_bvalid,
	s00_axi_bready => s_axi_bready,
	s00_axi_bresp => s_axi_bresp,
	s00_axi_araddr => s_axi_araddr,
	s00_axi_arprot => s_axi_arprot,
	s00_axi_arvalid => s_axi_arvalid,
	s00_axi_arready => s_axi_arready,
	s00_axi_rdata => s_axi_rdata,
	s00_axi_rvalid => s_axi_rvalid,
	s00_axi_rready => s_axi_rready,
	s00_axi_rresp => s_axi_rresp   
   );

   -- proces za urni takt
   s_axi_aclk_process :process
   begin
    if sim='1' then
        s_axi_aclk <= '0';
        wait for period/2;
        s_axi_aclk <= '1';
        wait for period/2;
    else
        wait;
    end if;
   end process;
 
   s_axi_aresetn <= '0', '1' after 2*period;

   -- stimulatorji za testiranje komponente
   stim_proc: process
   -- write32, sekvenca za pisanje na vodilo
   procedure write32(adr: integer; data: integer) is  -- postopek pisanja na AXI-lite
   begin
     wait until rising_edge(s_axi_aclk);
		s_axi_awaddr <= std_logic_vector(to_unsigned(adr, 4));  -- nastavi naslov in podatek
		s_axi_wdata <= std_logic_vector(to_unsigned(data, 32));
		s_axi_wstrb <= x"f";  -- 32 bitni zapis
		s_axi_awvalid <= '1'; -- aktiviraj kontrolne signale
		s_axi_wvalid <= '1';
		s_axi_bready <= '1';
     wait until rising_edge(s_axi_aclk) and s_axi_wready='1'; -- vpis
		s_axi_awvalid <= '0'; -- deaktiviraj kontrolne signale
		s_axi_wvalid <= '0';
     wait until rising_edge(s_axi_aclk) and s_axi_bvalid='1'; -- zakljucek
		s_axi_bready <= '0'; -- deaktiviraj kanal response
   end write32; 
  
   -- read32, sekvenca za branje vodila
   procedure read32(adr: integer) is
   begin
     wait until rising_edge(S_AXI_ACLK);
     S_AXI_ARADDR <= std_logic_vector(to_unsigned(adr, 4));  -- nastavi naslov
     S_AXI_ARVALID <= '1'; -- aktiviraj kontrolne signale za branje
     S_AXI_RREADY <= '1';            
     wait until falling_edge(S_AXI_ARREADY); -- cakaj zakljucek branja 
     report "s_axi_rdata=" & integer'image(to_integer(unsigned(s_axi_rdata)));   
     S_AXI_ARADDR <= x"0"; -- pobrisi naslov (ni potrebno)
     S_AXI_ARVALID <= '0'; -- deaktiviraj kontrolne signale    
     wait until falling_edge(S_AXI_RVALID);
     S_AXI_RREADY <= '0';
   end read32;  
   
   begin
      wait for 3*period; -- cakaj na reset
      write32(0, 20);    -- vpis v register r0, ... dopolni
      
      wait for 10*period; -- pocakaj in beri iz r3 ...
      read32(12);
      sim <= '0';
      wait;
   end process;

end Behavioral;
